@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2014 Yuichi Watanabe
+ * Copyright (c) 2014-2015 Yuichi Watanabe
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -10,7 +10,7 @@
  * 2. Redistributions in binary form must reproduce the above copyright notice,
  *    this list of conditions and the following disclaimer in the documentation
  *    and/or other materials provided with the distribution.
- * 3. Neither the name of Yuichi Watanabe nor the names of its
+ * 3. Neither the name of the copyright holder nor the names of its
  *    contributors may be used to endorse or promote products derived from
  *    this software without specific prior written permission.
  *
@@ -55,11 +55,12 @@
 #define FW_CFG_DATA		0x0511
 
 #define FW_CFG_SIGNATURE	0x0000
+#define FW_CFG_CPU_COUNT	0x0005
 #define FW_CFG_FILE_DIR		0x0019
 #define FW_CFG_FILE_START	0xc000
 
 #define FW_CFG_FILE_MAX_NUM	5
-#define FW_CFG_MAX_NUM		(FW_CFG_FILE_MAX_NUM + 2)
+#define FW_CFG_MAX_NUM		(FW_CFG_FILE_MAX_NUM + 3)
 
 struct cfg_file_info {
 	u32	size; /* big endian */
@@ -86,6 +87,7 @@ struct vm_fw_cfg {
 	u32			cfg_count;
 	struct fw_cfg		*cur_fw_cfg;
 	u32			cur_address;
+	u16			cpu_count;
 };
 
 static drvdata_hdl_t	fw_cfg_handle;
@@ -292,6 +294,14 @@ fw_cfg_setup(void)
 	if (err != VMMERR_SUCCESS) {
 		printf("Failed to register fw-cfg directory. %d\n", err);
 	}
+	vm_fw_cfg->cpu_count = vm_get_vcpu_count();
+	err = register_fw_cfg(vm_fw_cfg,
+			      FW_CFG_CPU_COUNT,
+			      (char *)&vm_fw_cfg->cpu_count,
+			      sizeof(vm_fw_cfg->cpu_count));
+	if (err != VMMERR_SUCCESS) {
+		printf("Failed to register cpu_count. %d\n", err);
+	}
 
 	register_files();
 
