@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2010-2014 Yuichi Watanabe
+ * Copyright (c) 2010-2015 Yuichi Watanabe
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -10,7 +10,7 @@
  * 2. Redistributions in binary form must reproduce the above copyright notice,
  *    this list of conditions and the following disclaimer in the documentation
  *    and/or other materials provided with the distribution.
- * 3. Neither the name of Yuichi Watanabe nor the names of its
+ * 3. Neither the name of the copyright holder nor the names of its
  *    contributors may be used to endorse or promote products derived from
  *    this software without specific prior written permission.
  *
@@ -74,8 +74,6 @@ gmm_alloc_avail(struct vm *vm, int count, struct resource *parent,
 {
 	struct resource *new;
 	phys_t size = *mem_size;
-	phys_t low_mem_size = ROUND_DOWN(mm_top_of_low_avail_mem() + 1,
-					 PAGESIZE);
 
 	while(count < MAXNUM_OF_SYSMEMMAP) {
 		new = vm->resource + count;
@@ -86,13 +84,6 @@ gmm_alloc_avail(struct vm *vm, int count, struct resource *parent,
 					    vm->name) != VMMERR_SUCCESS) {
 			break;
 		}
-		if (size <= low_mem_size) {
-			size += (new->end - new->start + 1);
-			if (size > low_mem_size) {
-				GMM_DBG("size > low_mem_size\n");
-				new->end -= (size - low_mem_size);
-			}
-		}
 		gmm_assign_reserve_mem(new->start, new->end);
 		GMM_DBG("%s %016llx-%016llx\n",
 			vm->name, new->start, new->end);
