@@ -67,6 +67,7 @@
 #define ALLOCLIST_DATASIZE(n)	((ALLOCLIST_DATABIT (n) + 7) / 8)
 #define ALLOCLIST_HEADERSIZE(n)	(sizeof (struct allocdata) + \
 				 ALLOCLIST_DATASIZE(n) - 1)
+#define NUM_OF_PANICMEM_PAGES	256
 
 #ifdef __x86_64__
 #	define PDPE_ATTR		(PDE_P_BIT | PDE_RW_BIT | PDE_US_BIT)
@@ -159,6 +160,7 @@ static u64 vmm_pd2[512] __attribute__ ((aligned (PAGESIZE)));
 static phys_t process_virt_to_phys_pdp_phys;
 static u64 *process_virt_to_phys_pdp;
 static u64 hphys_len;
+static int panicmem_start_page;
 
 static phys32_t top_of_low_avail_mem = 0;
 static phys_t top_of_avail_mem = 0;
@@ -541,6 +543,7 @@ mm_page_alloc (int n)
 	int s;
 	virt_t virt;
 	struct page *p, *q;
+	enum page_type old_type;
 
 	ASSERT (n < NUM_OF_ALLOCSIZE);
 	s = allocsize[n];
@@ -557,8 +560,16 @@ mm_page_alloc (int n)
 		LIST1_ADD (list1_freepage[n], p);
 		LIST1_ADD (list1_freepage[n], q);
 	}
-	spinlock_unlock (&mm_lock);
+	/* p->type must be set before unlock, because the
+	 * mm_page_free() function may merge blocks if the type is
+	 * PAGE_TYPE_FREE. */
+	old_type = p->type;
 	p->type = PAGE_TYPE_ALLOCATED;
+	spinlock_unlock (&mm_lock);
+	/* The old_type must be PAGE_TYPE_FREE, or the memory will be
+	 * corrupted.  The ASSERT is called after unlock to avoid
+	 * deadlocks during panic. */
+	ASSERT (old_type == PAGE_TYPE_FREE);
 	return p;
 }
 
@@ -912,18 +923,14 @@ mm_init_global (void)
 		pagestruct[i].phys = vmm_start_phys + PAGESIZE * i;
 		pagestruct[i].virt = VMM_START_VIRT + PAGESIZE * i;
 	}
+	panicmem_start_page = ((u64)(virt_t)end + PAGESIZE - 1 -
+			       VMM_START_VIRT) >> PAGESIZE_SHIFT;
 	for (i = 0; i < NUM_OF_PAGES; i++) {
 		if ((u64)(virt_t)head <= pagestruct[i].virt &&
 		    pagestruct[i].virt < (u64)(virt_t)end)
 			continue;
-#ifdef FWDBG
-		if (i == 0) {
-			extern char *dbgpage;
-			dbgpage = (char *)pagestruct[i].virt;
-			snprintf (dbgpage, PAGESIZE, "BitVisor dbgpage\n");
+		if (i < panicmem_start_page + NUM_OF_PANICMEM_PAGES)
 			continue;
-		}
-#endif
 		mm_page_free (&pagestruct[i]);
 	}
 	mapmem_lastvirt = MAPMEM_ADDR_START;
@@ -934,6 +941,32 @@ mm_init_global (void)
 	mm_init_mem_address_space();
 }
 
+/* panicmem is reserved memory for panic */
+void *
+mm_get_panicmem (int *len)
+{
+	int s = panicmem_start_page;
+
+	if (!s)
+		return NULL;
+	if (len)
+		*len = NUM_OF_PANICMEM_PAGES << PAGESIZE_SHIFT;
+	return (void *)pagestruct[s].virt;
+}
+
+void
+mm_free_panicmem (void)
+{
+	int s = panicmem_start_page;
+	int i;
+
+	if (!s)
+		return;
+	panicmem_start_page = 0;
+	for (i = 0; i < NUM_OF_PANICMEM_PAGES; i++)
+		mm_page_free (&pagestruct[s + i]);
+}
+
 /* allocate n or more pages */
 int
 alloc_pages (void **virt, u64 *phys, int n)
