@@ -86,6 +86,8 @@ svm_read_msr (u32 msrindex, u64 *msrdata)
 			data |= MSR_IA32_EFER_LME_BIT;
 		if (current->u.svm.lma)
 			data |= MSR_IA32_EFER_LMA_BIT;
+		if (current->u.svm.svme)
+			data |= MSR_IA32_EFER_SVME_BIT;
 		*msrdata = data;
 		break;
 	case MSR_IA32_STAR:
@@ -153,6 +155,12 @@ svm_read_msr (u32 msrindex, u64 *msrdata)
 	case MSR_AMD_TOP_MEM2:
 		r = cache_get_gmsr_amd (msrindex, msrdata);
 		break;
+	case MSR_AMD_VM_CR:
+		*msrdata = current->u.svm.vm_cr;
+		break;
+	case MSR_AMD_VM_HSAVE_PA:
+		*msrdata = current->u.svm.hsave_pa;
+		break;
 	default:
 		r = current->msr.read_msr (msrindex, msrdata);
 	}
@@ -179,7 +187,13 @@ svm_write_msr (u32 msrindex, u64 msrdata)
 		vmcb->sysenter_eip = msrdata;
 		break;
 	case MSR_IA32_EFER:
+		if ((current->u.svm.vm_cr & MSR_AMD_VM_CR_SVMDIS_BIT) &&
+		    (msrdata & MSR_IA32_EFER_SVME_BIT)) {
+			r = true;
+			break;
+		}
 		current->u.svm.lme = !!(msrdata & MSR_IA32_EFER_LME_BIT);
+		current->u.svm.svme = !!(msrdata & MSR_IA32_EFER_SVME_BIT);
 		vmcb->efer = (vmcb->efer & mask) | (msrdata & ~mask);
 		/* FIXME: Reserved bits should be checked here. */
 		svm_msr_update_lma ();
@@ -207,7 +221,14 @@ svm_write_msr (u32 msrindex, u64 msrdata)
 		vmcb->kernel_gs_base = msrdata;
 		break;
 	case MSR_AMD_VM_CR:
+		current->u.svm.vm_cr =
+			(current->u.svm.vm_cr & (MSR_AMD_VM_CR_LOCK_BIT |
+						 MSR_AMD_VM_CR_SVMDIS_BIT)) |
+			(msrdata & ~(MSR_AMD_VM_CR_LOCK_BIT |
+				     MSR_AMD_VM_CR_SVMDIS_BIT));
+		break;
 	case MSR_AMD_VM_HSAVE_PA:
+		current->u.svm.hsave_pa = msrdata;
 		break;
 	case MSR_IA32_MTRR_FIX4K_C0000:
 	case MSR_IA32_MTRR_FIX4K_C8000:
@@ -258,3 +279,94 @@ svm_write_msr (u32 msrindex, u64 msrdata)
 	}
 	return r;
 }
+
+static void
+svm_setmsrbmp (u8 *p, u32 bit2offset, bool wr, int bit)
+{
+	u32 bitoffset = bit2offset << 1;
+
+	if (wr)
+		bitoffset++;
+	if (bit)
+		p[bitoffset >> 3] |= 1 << (bitoffset & 7);
+	else
+		p[bitoffset >> 3] &= ~(1 << (bitoffset & 7));
+}
+
+void
+svm_msrpass (u32 msrindex, bool wr, bool pass)
+{
+	u8 *p;
+
+	switch (msrindex) {
+	case MSR_IA32_SYSENTER_CS:
+	case MSR_IA32_SYSENTER_ESP:
+	case MSR_IA32_SYSENTER_EIP:
+	case MSR_IA32_STAR:
+	case MSR_IA32_LSTAR:
+	case MSR_AMD_CSTAR:
+	case MSR_IA32_FMASK:
+	case MSR_IA32_FS_BASE:
+	case MSR_IA32_GS_BASE:
+	case MSR_IA32_KERNEL_GS_BASE:
+		/* These are able to be pass-through. */
+		break;
+	case MSR_AMD_SYSCFG:
+	case MSR_AMD_TOP_MEM2:
+	case MSR_IA32_EFER:
+	case MSR_IA32_MTRR_DEF_TYPE:
+	case MSR_IA32_MTRR_FIX16K_80000:
+	case MSR_IA32_MTRR_FIX16K_A0000:
+	case MSR_IA32_MTRR_FIX4K_C0000:
+	case MSR_IA32_MTRR_FIX4K_C8000:
+	case MSR_IA32_MTRR_FIX4K_D0000:
+	case MSR_IA32_MTRR_FIX4K_D8000:
+	case MSR_IA32_MTRR_FIX4K_E0000:
+	case MSR_IA32_MTRR_FIX4K_E8000:
+	case MSR_IA32_MTRR_FIX4K_F0000:
+	case MSR_IA32_MTRR_FIX4K_F8000:
+	case MSR_IA32_MTRR_FIX64K_00000:
+	case MSR_IA32_MTRR_PHYSBASE0:
+	case MSR_IA32_MTRR_PHYSBASE1:
+	case MSR_IA32_MTRR_PHYSBASE2:
+	case MSR_IA32_MTRR_PHYSBASE3:
+	case MSR_IA32_MTRR_PHYSBASE4:
+	case MSR_IA32_MTRR_PHYSBASE5:
+	case MSR_IA32_MTRR_PHYSBASE6:
+	case MSR_IA32_MTRR_PHYSBASE7:
+	case MSR_IA32_MTRR_PHYSBASE8:
+	case MSR_IA32_MTRR_PHYSBASE9:
+	case MSR_IA32_MTRR_PHYSMASK0:
+	case MSR_IA32_MTRR_PHYSMASK1:
+	case MSR_IA32_MTRR_PHYSMASK2:
+	case MSR_IA32_MTRR_PHYSMASK3:
+	case MSR_IA32_MTRR_PHYSMASK4:
+	case MSR_IA32_MTRR_PHYSMASK5:
+	case MSR_IA32_MTRR_PHYSMASK6:
+	case MSR_IA32_MTRR_PHYSMASK7:
+	case MSR_IA32_MTRR_PHYSMASK8:
+	case MSR_IA32_MTRR_PHYSMASK9:
+	case MSR_IA32_PAT:
+		pass = false;
+		break;
+	case MSR_IA32_MTRRCAP:
+		if (!wr)
+			pass = false;
+		break;
+	case MSR_AMD_VM_CR:
+	case MSR_AMD_VM_HSAVE_PA:
+		pass = false;
+		break;
+	case 0xC0010020:	/* PATCH_LOADER MSR */
+		if (wr)
+			pass = true;
+		break;
+	}
+	p = current->vm->u.svm.msrbmp->msrbmp;
+	if (msrindex <= 0x1FFF)
+		svm_setmsrbmp (p, msrindex, wr, !pass);
+	else if (msrindex >= 0xC0000000 && msrindex <= 0xC0001FFF)
+		svm_setmsrbmp (p + 0x800, msrindex - 0xC0000000, wr, !pass);
+	else if (msrindex >= 0xC0010000 && msrindex <= 0xC0011FFF)
+		svm_setmsrbmp (p + 0x1000, msrindex - 0xC0010000, wr, !pass);
+}
