@@ -59,6 +59,7 @@
 #include "vt_vmcs.h"
 
 #define EPT_VIOLATION_EXIT_QUAL_WRITE_BIT 0x2
+#define STAT_EXIT_REASON_MAX EXIT_REASON_XSETBV
 
 enum vt__status {
 	VT__VMENTRY_SUCCESS,
@@ -72,6 +73,7 @@ static u32 stat_swexcnt = 0;
 static u32 stat_pfcnt = 0;
 static u32 stat_iocnt = 0;
 static u32 stat_hltcnt = 0;
+static u32 stat_exit_reason[STAT_EXIT_REASON_MAX + 1];
 
 static void
 do_mov_cr (void)
@@ -162,6 +164,53 @@ vt_update_exception_bmp (void)
 	asm_vmwrite (VMCS_EXCEPTION_BMP, newbmp);
 }
 
+static void
+vt_generate_nmi (void)
+{
+	struct vt_intr_data *vid = &current->u.vt.intr;
+
+	if (current->u.vt.vr.re)
+		panic ("NMI in real mode");
+	vid->vmcs_intr_info.v = 0;
+	vid->vmcs_intr_info.s.vector = EXCEPTION_NMI;
+	vid->vmcs_intr_info.s.type = INTR_INFO_TYPE_NMI;
+	vid->vmcs_intr_info.s.err = INTR_INFO_ERR_INVALID;
+	vid->vmcs_intr_info.s.valid = INTR_INFO_VALID_VALID;
+	vid->vmcs_instruction_len = 0;
+}
+
+/* NMI handler.  FIXME: This is currently pass-through only. */
+static void
+vt_nmi_has_come (void)
+{
+	ulong is, proc_based_vmexec_ctl;
+
+	/* If blocking by NMI bit is set, the NMI will not be
+	 * generated since an NMI handler in the guest operating
+	 * system is running. */
+	asm_vmread (VMCS_GUEST_INTERRUPTIBILITY_STATE, &is);
+	if (is & VMCS_GUEST_INTERRUPTIBILITY_STATE_BLOCKING_BY_NMI_BIT)
+		return;
+	/* If NMI-window exiting bit is set, VM Exit reason "NMI
+	   window" will generate NMI. */
+	asm_vmread (VMCS_PROC_BASED_VMEXEC_CTL, &proc_based_vmexec_ctl);
+	if (proc_based_vmexec_ctl & VMCS_PROC_BASED_VMEXEC_CTL_NMIWINEXIT_BIT)
+		return;
+	/* If blocking by STI bit and blocking by MOV SS bit are not
+	   set, generate NMI now. */
+	if (!(is & VMCS_GUEST_INTERRUPTIBILITY_STATE_BLOCKING_BY_STI_BIT) &&
+	    !(is & VMCS_GUEST_INTERRUPTIBILITY_STATE_BLOCKING_BY_MOV_SS_BIT)) {
+		vt_generate_nmi ();
+		return;
+	}
+	/* Use NMI-window exiting to get the correct timing to inject
+	 * NMIs.  This is a workaround for a processor that makes a VM
+	 * Entry failure when NMI is injected while blocking by STI
+	 * bit is set. */
+	proc_based_vmexec_ctl |= VMCS_PROC_BASED_VMEXEC_CTL_NMIWINEXIT_BIT;
+	asm_vmwrite (VMCS_PROC_BASED_VMEXEC_CTL, proc_based_vmexec_ctl);
+}
+
 static void
 do_exception (void)
 {
@@ -252,10 +301,7 @@ do_exception (void)
 			current->u.vt.intr.vmcs_instruction_len = len;
 			break;
 		case INTR_INFO_TYPE_NMI:
-			vii.s.nmi = 0; /* FIXME */
-			current->u.vt.intr.vmcs_intr_info.v = vii.v;
-			asm_vmread (VMCS_VMEXIT_INSTRUCTION_LEN, &len);
-			current->u.vt.intr.vmcs_instruction_len = len;
+			vt_nmi_has_come ();
 			break;
 		case INTR_INFO_TYPE_EXTERNAL:
 		default:
@@ -284,6 +330,23 @@ do_vmcall (void)
 	vmmcall ();
 }
 
+static void
+do_nmi_window (void)
+{
+	struct vt_intr_data *vid = &current->u.vt.intr;
+	ulong proc_based_vmexec_ctl;
+
+	if (vid->vmcs_intr_info.s.valid == INTR_INFO_VALID_VALID) {
+		/* This may be incorrect behavior... */
+		printf ("Maskable interrupt and NMI at the same time\n");
+		return;
+	}
+	asm_vmread (VMCS_PROC_BASED_VMEXEC_CTL, &proc_based_vmexec_ctl);
+	proc_based_vmexec_ctl &= ~VMCS_PROC_BASED_VMEXEC_CTL_NMIWINEXIT_BIT;
+	asm_vmwrite (VMCS_PROC_BASED_VMEXEC_CTL, proc_based_vmexec_ctl);
+	vt_generate_nmi ();
+}
+
 static void
 vt__nmi (void)
 {
@@ -293,13 +356,7 @@ vt__nmi (void)
 		return;
 	if (!current->nmi.get_nmi_count ())
 		return;
-	if (current->u.vt.vr.re)
-		panic ("NMI in real mode");
-	vid->vmcs_intr_info.v = 0;
-	vid->vmcs_intr_info.s.vector = EXCEPTION_NMI;
-	vid->vmcs_intr_info.s.type = INTR_INFO_TYPE_NMI;
-	vid->vmcs_intr_info.s.err = INTR_INFO_ERR_INVALID;
-	vid->vmcs_intr_info.s.valid = INTR_INFO_VALID_VALID;
+	vt_nmi_has_come ();
 }
 
 static void
@@ -338,11 +395,14 @@ static void
 vt__vm_run_first (void)
 {
 	enum vt__status status;
+	ulong errnum;
 
 	status = call_vt__vmlaunch ();
 	if (status != VT__VMEXIT) {
+		asm_vmread (VMCS_VM_INSTRUCTION_ERR, &errnum);
 		if (status == VT__VMENTRY_FAILED)
-			panic ("Fatal error: VM entry failed.");
+			panic ("Fatal error: VM entry failed. Error %lu",
+			       errnum);
 		else
 			panic ("Fatal error: Strange status.");
 	}
@@ -352,6 +412,7 @@ static void
 vt__vm_run (void)
 {
 	enum vt__status status;
+	ulong errnum;
 
 	if (current->u.vt.first) {
 		vt__vm_run_first ();
@@ -366,8 +427,10 @@ vt__vm_run (void)
 	if (current->u.vt.saved_vmcs)
 		spinlock_lock (&currentcpu->suspend_lock);
 	if (status != VT__VMEXIT) {
+		asm_vmread (VMCS_VM_INSTRUCTION_ERR, &errnum);
 		if (status == VT__VMENTRY_FAILED)
-			panic ("Fatal error: VM entry failed.");
+			panic ("Fatal error: VM entry failed. Error %lu",
+			       errnum);
 		else
 			panic ("Fatal error: Strange status.");
 	}
@@ -388,6 +451,16 @@ vt__vm_run_with_tf (void)
 	vt_write_flags (rflags);
 }
 
+static void
+clear_blocking_by_nmi (void)
+{
+	ulong is;
+
+	asm_vmread (VMCS_GUEST_INTERRUPTIBILITY_STATE, &is);
+	is &= ~VMCS_GUEST_INTERRUPTIBILITY_STATE_BLOCKING_BY_NMI_BIT;
+	asm_vmwrite (VMCS_GUEST_INTERRUPTIBILITY_STATE, is);
+}
+
 static void
 vt__event_delivery_check (void)
 {
@@ -415,6 +488,11 @@ vt__event_delivery_check (void)
 		} else if (ivif.s.err == INTR_INFO_ERR_VALID) {
 			asm_vmread (VMCS_IDT_VECTORING_ERRCODE, &err);
 			vid->vmcs_exception_errcode = err;
+		} else if (ivif.s.type == INTR_INFO_TYPE_NMI) {
+			/* If EPT violation happened during injecting
+			 * NMI, blocking by NMI bit is set.  It must
+			 * be cleared before injecting NMI again. */
+			clear_blocking_by_nmi ();
 		}
 	}
 	vid->vmcs_intr_info.v = ivif.v;
@@ -830,11 +908,19 @@ vt__exit_reason (void)
 	case EXIT_REASON_EPT_VIOLATION:
 		do_ept_violation ();
 		break;
+	case EXIT_REASON_NMI_WINDOW:
+		do_nmi_window ();
+		break;
 	default:
 		printf ("Fatal error: handler not implemented.\n");
 		printexitreason (exit_reason);
 		panic ("Fatal error: handler not implemented.");
 	}
+	STATUS_UPDATE (asm_lock_incl
+		       (&stat_exit_reason
+			[(exit_reason & EXIT_REASON_MASK) >
+			 STAT_EXIT_REASON_MAX ? STAT_EXIT_REASON_MAX :
+			 (exit_reason & EXIT_REASON_MASK)]));
 }
 
 static void
@@ -994,6 +1080,7 @@ vt_mainloop (void)
 		if (current->u.vt.vr.sw.enable) {
 			vt__nmi ();
 			vt__event_delivery_setup ();
+			vt_msr_own_process_msrs ();
 			vt__vm_run_with_tf ();
 			vt_paging_tlbflush ();
 			vt__event_delivery_check ();
@@ -1001,6 +1088,7 @@ vt_mainloop (void)
 		} else {	/* not switching */
 			vt__nmi ();
 			vt__event_delivery_setup ();
+			vt_msr_own_process_msrs ();
 			vt__vm_run ();
 			vt_paging_tlbflush ();
 			vt__event_delivery_check ();
@@ -1012,9 +1100,32 @@ vt_mainloop (void)
 static char *
 vt_status (void)
 {
-	static char buf[1024];
+	static char buf[4096];
+	int i, n;
 
-	snprintf (buf, 1024,
+	n = snprintf (buf, 4096, "Exit Reason:\n");
+	for (i = 0; i + 7 <= STAT_EXIT_REASON_MAX; i += 8) {
+		n += snprintf
+			(buf + n, 4096 - n,
+			 " %02X: %04X %04X %04X %04X %04X %04X %04X %04X\n",
+			 i, stat_exit_reason[i + 0] & 0xFFFF,
+			 stat_exit_reason[i + 1] & 0xFFFF,
+			 stat_exit_reason[i + 2] & 0xFFFF,
+			 stat_exit_reason[i + 3] & 0xFFFF,
+			 stat_exit_reason[i + 4] & 0xFFFF,
+			 stat_exit_reason[i + 5] & 0xFFFF,
+			 stat_exit_reason[i + 6] & 0xFFFF,
+			 stat_exit_reason[i + 7] & 0xFFFF);
+	}
+	if (i <= STAT_EXIT_REASON_MAX) {
+		n += snprintf (buf + n, 4096 - n, " %02X:", i);
+		for (; i < STAT_EXIT_REASON_MAX; i++)
+			n += snprintf (buf + n, 4096 - n, " %04X",
+				       stat_exit_reason[i] & 0xFFFF);
+		n += snprintf (buf + n, 4096 - n, " %04X\n",
+			       stat_exit_reason[i] & 0xFFFF);
+	}
+	snprintf (buf + n, 4096 - n,
 		  "Interrupts: %u\n"
 		  "Hardware exceptions: %u\n"
 		  " Page fault: %u\n"
