@@ -26,11 +26,15 @@
  * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
  * POSSIBILITY OF SUCH DAMAGE.
  */
+/*
+ * Copyright (c) 2015 Yuichi Watanabe
+ */
 
 #include "asm.h"
 #include "constants.h"
 #include "current.h"
 #include "entry.h"
+#include "gbiosloader.h"
 #include "mm.h"
 #include "panic.h"
 #include "printf.h"
@@ -172,16 +176,16 @@ vt_read_control_reg (enum control_reg reg, ulong *val)
 {
 	switch (reg) {
 	case CONTROL_REG_CR0:
-		asm_vmread (VMCS_CR0_READ_SHADOW, val);
+		*val = vt_read_cr0 ();
 		break;
 	case CONTROL_REG_CR2:
 		*val = current->u.vt.vr.cr2;
 		break;
 	case CONTROL_REG_CR3:
-		*val = current->u.vt.vr.cr3;
+		*val = vt_read_cr3 ();
 		break;
 	case CONTROL_REG_CR4:
-		asm_vmread (VMCS_CR4_READ_SHADOW, val);
+		*val = vt_read_cr4 ();
 		break;
 	default:
 		panic ("Fatal error: unknown control register.");
@@ -375,7 +379,7 @@ vt_write_control_reg (enum control_reg reg, ulong val)
 		current->u.vt.vr.cr2 = val;
 		break;
 	case CONTROL_REG_CR3:
-		current->u.vt.vr.cr3 = val;
+		vt_write_cr3 (val);
 		vt_paging_updatecr3 ();
 		vt_paging_flush_guest_tlb ();
 		break;
@@ -783,18 +787,22 @@ vt_reset (void)
 	asm_vmwrite (VMCS_CR0_READ_SHADOW, CR0_ET_BIT);
 	asm_vmwrite (VMCS_GUEST_CR0, vt_paging_apply_fixed_cr0 (CR0_ET_BIT));
 	current->u.vt.vr.cr2 = 0;
-	current->u.vt.vr.cr3 = 0;
+	vt_write_cr3 (0);
 	asm_vmwrite (VMCS_CR4_READ_SHADOW, 0);
 	asm_vmwrite (VMCS_GUEST_CR4, vt_paging_apply_fixed_cr4 (0));
 	vt_write_msr (MSR_IA32_EFER, 0);
 	asm_vmwrite (VMCS_GUEST_ES_SEL, 0);
-	asm_vmwrite (VMCS_GUEST_CS_SEL, 0);
+	asm_vmwrite (VMCS_GUEST_CS_SEL, 0xf000);
 	asm_vmwrite (VMCS_GUEST_SS_SEL, 0);
 	asm_vmwrite (VMCS_GUEST_DS_SEL, 0);
 	asm_vmwrite (VMCS_GUEST_FS_SEL, 0);
 	asm_vmwrite (VMCS_GUEST_GS_SEL, 0);
 	asm_vmwrite (VMCS_GUEST_ES_BASE, 0);
-	asm_vmwrite (VMCS_GUEST_CS_BASE, 0);
+	if (guest_bios_type() == BIOS_TYPE_UEFI) {
+		asm_vmwrite (VMCS_GUEST_CS_BASE, 0xffff0000);
+	} else {
+		asm_vmwrite (VMCS_GUEST_CS_BASE, 0xf0000);
+	}
 	asm_vmwrite (VMCS_GUEST_SS_BASE, 0);
 	asm_vmwrite (VMCS_GUEST_DS_BASE, 0);
 	asm_vmwrite (VMCS_GUEST_FS_BASE, 0);
