@@ -33,6 +33,7 @@
 #include <common.h>
 #include <core.h>
 #include <core/mm.h>
+#include <core/mmio.h>
 #include <core/gmm.h>
 #include <core/process.h>
 #include <core/panic.h>
@@ -140,6 +141,7 @@
 
 struct iommu {
 	LIST_DEFINE(iommu_list) ;
+        phys_t reg_phys;
         void *reg;    /* register base address of the unit = drhd->address */
         u32 gcmd;             /* maintaining TE field */
         u64 cap;              /* capability register */
@@ -835,6 +837,7 @@ new_iommu(phys_t reg_base)
 
 	memset(iommu, 0, sizeof(struct iommu));
 
+	iommu->reg_phys = reg_base;
 	iommu->reg = mapmem_hphys(reg_base, PAGE_SIZE,
 				  MAPMEM_WRITE | MAPMEM_UC);
 	if (iommu->reg == NULL) {
@@ -972,7 +975,7 @@ vtd_pci_new(struct pci_device *dev)
 }
 
 struct pci_driver vtd_pci_driver = {
-	.id = {0xD1558086, 0xFFFFFFFF},
+	.device = "id=8086:d155",
 	.new = vtd_pci_new,
 	.name = "vtd_pci",
 	.longname = "VT-d PCI driver"
@@ -997,6 +1000,7 @@ iommu_init(void)
 
 	clflush_size = ((cpuid_ebx(1) >> 8) & 0xff) * 8;
 
+	dmar_init();
 	if (dmar_drhd_exists()) {
 		for (drhd = dmar_get_next_drhd(NULL); drhd;
 		     drhd = dmar_get_next_drhd(drhd)) {
@@ -1016,11 +1020,22 @@ static void
 iommu_initvm(void)
 {
 	struct io_domain *dom;
+	struct iommu *iommu;
 
 	if (!LIST_HEAD(iommu_list)) {
 		return;
 	}
 
+	if (vm_get_id() == 0) {
+		/*
+		 * Hide iommu from OS.
+		 */
+		LIST_FOREACH(iommu_list, iommu) {
+			mmio_register(iommu->reg_phys, PAGESIZE,
+				      mmio_do_nothing, NULL);
+		}
+	}
+
 	dom = create_domain();
 	if (!dom) {
 		panic("Failed to create domain.");
@@ -1053,6 +1068,6 @@ iommu_enable(void)
 	printf("VT-d IOMMU enabled.\n");
 }
 
-VTD_INIT(iommu_init);
+DRIVER_INIT(iommu_init);
 DRIVER_VMINIT(iommu_initvm);
 DRIVER_START(iommu_enable);
