@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2010-2014 Yuichi Watanabe
+ * Copyright (c) 2010-2015 Yuichi Watanabe
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -10,7 +10,7 @@
  * 2. Redistributions in binary form must reproduce the above copyright notice,
  *    this list of conditions and the following disclaimer in the documentation
  *    and/or other materials provided with the distribution.
- * 3. Neither the name of Yuichi Watanabe nor the names of its
+ * 3. Neither the name of the copyright holder nor the names of its
  *    contributors may be used to endorse or promote products derived from
  *    this software without specific prior written permission.
  *
@@ -55,13 +55,16 @@
 #define PCIASSIGN_DBG(...)
 #endif
 
-#define PCI_DUMMY_FUNC0_VENDOR_ID 0x0001
+#define PCI_DUMMY_FUNC0_VENDOR_ID 0x0002
 #define PCI_DUMMY_FUNC0_DEVICE_ID 0x0001
 
 #define DUMP_ASSIGNMENT
 /* #define NO_CLEAR_IOBITMAP */
+
 /* #define NO_SECONDARY_BUS_RESET */
 /* #define NO_D3HOT_D0_RESET */
+/* #define NO_REINIT_DEVICE */
+/* #define REINIT_ALL_REGS */
 
 drvdata_hdl_t		pci_handle;
 
@@ -71,14 +74,24 @@ pci_config_size(struct pci_device *dev)
 	return dev->config_mmio ? PCI_MMCONFIG_SIZE : PCI_LEGACY_CONFIG_SIZE;
 }
 
+void
+pci_update_saved_config(struct pci_device *dev, pci_off_t offset,
+			void *val, uint iosize)
+{
+	char *config;
+
+	config = (char *)dev->saved_config;
+	memcpy(config + offset, val, iosize);
+}
+
 static void
 pci_save_config(struct pci_device *dev)
 {
 	int size = pci_config_size(dev);
 	int i;
 
-	for (i = 0; i < size; i += sizeof(u32)) {
-		dev->saved_config[i / sizeof(u32)] = pci_read_config32(dev, i);
+	for (i = 0; i < size; i += 4) {
+		dev->saved_config[i / 4] = pci_read_config32(dev, i);
 	}
 }
 
@@ -573,6 +586,15 @@ pci_secondary_bus_reset(struct pci_device *dev)
 {
 #ifndef NO_SECONDARY_BUS_RESET
 	u16 reg;
+	u16 vendor_id;
+	u16 device_id;
+
+	vendor_id = pci_read_config16(dev, PCI_CONFIG_VENDOR_ID);
+	device_id = pci_read_config16(dev, PCI_CONFIG_DEVICE_ID);
+	if (dev->vendor_id != vendor_id || dev->device_id != device_id) {
+		/* A device might be replaced */
+		return;
+	}
 
 	if (dev->type != PCI_CONFIG_HEADER_TYPE_1) {
 		return;
@@ -596,10 +618,6 @@ pci_d3d0_reset_device(struct pci_device *dev)
 	pci_off_t base;
 	u16 reg;
 
-	if (dev->type != PCI_CONFIG_HEADER_TYPE_0) {
-		return;
-	}
-
 	base = pci_find_capability(dev, PCI_PM_CAP_ID);
 	if (base == 0) {
 		return;
@@ -632,6 +650,15 @@ pci_d3d0_reset_devices(struct pci_device *dev)
 {
 	struct pci_device *child;
 	bool has_child = false;
+	u16 vendor_id;
+	u16 device_id;
+
+	vendor_id = pci_read_config16(dev, PCI_CONFIG_VENDOR_ID);
+	device_id = pci_read_config16(dev, PCI_CONFIG_DEVICE_ID);
+	if (dev->vendor_id != vendor_id || dev->device_id != device_id) {
+		/* A device might be replaced */
+		return;
+	}
 
 	LIST2_FOREACH(dev->children, sibling, child) {
 		pci_d3d0_reset_devices(child);
@@ -644,36 +671,107 @@ pci_d3d0_reset_devices(struct pci_device *dev)
 }
 
 static void
-pci_reprogram_device(struct pci_device *dev)
+pci_reinit_device(struct pci_device *dev)
 {
+#ifndef NO_REINIT_DEVICE
+#ifdef REINIT_ALL_REGS
 	int size = pci_config_size(dev);
 	int i;
 
-	printf("Reprogram %s\n", dev->name);
-	for (i = 0; i < size; i += sizeof(u32)) {
-		pci_write_config32(dev, i, dev->saved_config[i / sizeof(u32)]);
+	printf("Reinit %s\n", dev->name);
+	pci_write_config32(dev, PCI_CONFIG_COMMAND, 0x0);
+	for (i = PCI_CONFIG_COMMAND + 4; i < size; i += 4) {
+		pci_write_config32(dev, i, dev->saved_config[i / 4]);
 	}
+	pci_write_config32(dev, PCI_CONFIG_COMMAND,
+		   dev->saved_config[PCI_CONFIG_COMMAND / 4]);
+
+#else /* REINIT_ALL_REGS */
+
+	pci_off_t base, size;
+	int i;
+	u16 reg;
+
+	printf("Reinit %s\n", dev->name);
+	pci_write_config32(dev, PCI_CONFIG_COMMAND, 0x0);
+
+	for (i = PCI_CONFIG_COMMAND + 4; i < PCI_CONFIG_HEADER_SIZE; i += 4) {
+		pci_write_config32(dev, i, dev->saved_config[i / 4]);
+	}
+
+	base = pci_find_capability(dev, PCI_PM_CAP_ID);
+	if (base != 0) {
+		size = PCI_PM_CAP_STRUCTURE_SIZE;
+		for (i = base; i < base + size; i += 4) {
+			pci_write_config32(dev, i, dev->saved_config[i / 4]);
+		}
+	}
+
+	base = pci_find_capability(dev, PCI_PCIE_CAP_ID);
+	if (base != 0) {
+		reg = pci_read_config16(dev, base + PCI_PCIE_CAP_REG_OFF);
+		if ((reg & PCI_PCIE_CAP_REG_PCIECV_MASK) >= 2) {
+			size = PCI_PCIE_CAP_REV2_STRUCTURE_SIZE;
+		} else {
+			size = PCI_PCIE_CAP_REV1_STRUCTURE_SIZE;
+		}
+		for (i = base; i < base + size; i += 4) {
+			pci_write_config32(dev, i, dev->saved_config[i / 4]);
+		}
+	}
+
+	pci_write_config32(dev, PCI_CONFIG_COMMAND,
+			   dev->saved_config[PCI_CONFIG_COMMAND / 4]);
+#endif /* !REINIT_ALL_REGS */
+#endif /* NO_REINIT_DEVICE */
 }
 
 static void
-pci_reprogram_devices(struct pci_device *dev)
+pci_reinit_devices(struct pci_device *dev, bool root_bus)
 {
 	struct pci_device *child;
+	pci_off_t base;
+	u16 reg;
 	u16 vendor_id;
 	u16 device_id;
 
 	vendor_id = pci_read_config16(dev, PCI_CONFIG_VENDOR_ID);
 	device_id = pci_read_config16(dev, PCI_CONFIG_DEVICE_ID);
-
 	if (dev->vendor_id != vendor_id || dev->device_id != device_id) {
 		/* A device might be replaced */
 		return;
 	}
 
-	pci_reprogram_device(dev);
+	if (root_bus &&
+	    LIST2_NEXT(dev->children, sibling, (struct pci_device *)NULL)
+		== NULL) {
+		base = pci_find_capability(dev, PCI_PM_CAP_ID);
+		if (base != 0) {
+			reg = pci_read_config16(dev,
+				base + PCI_PM_CONTROL_STATUS_REG_OFF);
+			if ((reg & PCI_PM_CONTROL_STATUS_REG_NO_SOFT_RESET)
+			    == 0) {
+				/*
+				 * VMM re-init device on root bus if
+				 * it is reset by d3 to d0 reset.
+				 */
+				pci_reinit_device(dev);
+			}
+		}
+	} else if (!root_bus) {
+		/*
+		 * VMM re-init device on non root bus which is reset
+		 * by secondary bus reset and d3 to d0 reset.
+		 */
+		pci_reinit_device(dev);
+	}
+
+	if (dev->driver && dev->driver->reinit) {
+		dev->driver->reinit(dev);
+	}
 
 	LIST2_FOREACH(dev->children, sibling, child) {
-		pci_reprogram_devices(child);
+		pci_reinit_devices(child, false);
 	}
 
 }
@@ -702,9 +800,9 @@ pci_reset_assigned_device(void)
 	/* wait 2 sec */
 	usleep(2 * 1000 * 1000);
 
-	/* Reprogram devices */
+	/* Reinit devices */
 	LIST2_FOREACH(pci_data->vm_pci_device_list, vm_pci_device_list, dev) {
-		pci_reprogram_devices(dev);
+		pci_reinit_devices(dev, true);
 	}
 
 	/* wait 2 sec */
