@@ -37,6 +37,9 @@
 
 typedef u8 spinlock_t;
 typedef u32 rw_spinlock_t;
+typedef struct {
+	u32 next_ticket, now_serving;
+} ticketlock_t;
 
 #define SPINLOCK_INITIALIZER ((spinlock_t)0)
 
@@ -205,4 +208,48 @@ rw_spinlock_init (rw_spinlock_t *l)
 	*l = 0;
 }
 
+static inline void
+ticketlock_lock (ticketlock_t *l)
+{
+	u32 ticket;
+
+	asm volatile (" lock cmpxchgl %1, %0 \n" /* a=next */
+		      "      mov      %1, %2 \n" /* r=a */
+		      "1: \n"
+		      "      add      $1, %2 \n" /* r++ */
+		      " lock cmpxchgl %2, %0 \n" /* if next==a then next=r; */
+		      "      mov      %1, %2 \n" /* r=a */
+		      "      je       1f \n"	 /* ;goto 1f */
+		      "      pause \n"		 /* spin loop hint */
+		      "      jmp 1b \n"		 /* do spin loop */
+		      "1: \n"
+		      " lock cmpxchgl %1, %3 \n" /* if now==a */
+		      "      je       1f \n"	 /* then return */
+		      "      mov      %2, %1 \n" /* a=r */
+		      "      pause \n"		 /* spin loop hint */
+		      "      jmp 1b \n"		 /* do spin loop */
+		      "1: \n"
+		      : "+m" (l->next_ticket)
+		      , "=&a" (ticket)
+		      , "=&r" (ticket)
+		      : "m" (l->now_serving)
+		      : "cc");
+}
+
+static inline void
+ticketlock_unlock (ticketlock_t *l)
+{
+	asm volatile ("lock addl $1, %0"
+		      : "+m" (l->now_serving)
+		      :
+		      : "cc");
+}
+
+static inline void
+ticketlock_init (ticketlock_t *l)
+{
+	l->next_ticket = 0;
+	l->now_serving = 0;
+}
+
 #endif
